

#include "gpio.h"
#include "glcd_s6b0108.h"

//----------------Stałe----------------------//
#define LCD_CMD 0 
#define LCD_DATA 1
#define LCD_47_U (SystemCoreClock/25000)/5
#define LCD_1_M_5 (SystemCoreClock/500)/5
#define LCD_1_U (SystemCoreClock/1000000)/5
#define LCD_500_N (SystemCoreClock/2000000)/5
#define LCD_100_N (SystemCoreClock/10000000)/5
#define LCD_PAGE_SIZE 64
#define LCD_NUM_PAGES 8

/* 
CAUTION! 
Display is divided into 3 independently controlled sections of 64x64 pixels.
Each section is accesed through control interface by setting
two chip  select lines , CSA and CSB, in this order:

left section:   CSA=0, CSB=0
middle section: CSA=1, CSB=0
right section:  CSA=0, CSB=1

CSA=1, CSB=1 does nothing
*/

#define LCD_LEFT_SECTION CSA|CSB
#define LCD_MID_SECTION CSB
#define LCD_RIGHT_SECTION CSA

//komendy LCD
#define LCD_DISP_ON 0x3F
#define LCD_DISP_OFF 0x3E
#define LCD_SET_X_ADDR 0xB8
#define LCD_SET_Y_ADDR 0x40
#define LCD_SET_Z_ADDR 0x40

               		

//-----------------------------Zmienne globalne------------------------------//
static const unsigned char BitReverseTable256[] = 
{
  0x00, 0x80, 0x40, 0xC0, 0x20, 0xA0, 0x60, 0xE0, 0x10, 0x90, 0x50, 0xD0, 0x30, 0xB0, 0x70, 0xF0, 
  0x08, 0x88, 0x48, 0xC8, 0x28, 0xA8, 0x68, 0xE8, 0x18, 0x98, 0x58, 0xD8, 0x38, 0xB8, 0x78, 0xF8, 
  0x04, 0x84, 0x44, 0xC4, 0x24, 0xA4, 0x64, 0xE4, 0x14, 0x94, 0x54, 0xD4, 0x34, 0xB4, 0x74, 0xF4, 
  0x0C, 0x8C, 0x4C, 0xCC, 0x2C, 0xAC, 0x6C, 0xEC, 0x1C, 0x9C, 0x5C, 0xDC, 0x3C, 0xBC, 0x7C, 0xFC, 
  0x02, 0x82, 0x42, 0xC2, 0x22, 0xA2, 0x62, 0xE2, 0x12, 0x92, 0x52, 0xD2, 0x32, 0xB2, 0x72, 0xF2, 
  0x0A, 0x8A, 0x4A, 0xCA, 0x2A, 0xAA, 0x6A, 0xEA, 0x1A, 0x9A, 0x5A, 0xDA, 0x3A, 0xBA, 0x7A, 0xFA,
  0x06, 0x86, 0x46, 0xC6, 0x26, 0xA6, 0x66, 0xE6, 0x16, 0x96, 0x56, 0xD6, 0x36, 0xB6, 0x76, 0xF6, 
  0x0E, 0x8E, 0x4E, 0xCE, 0x2E, 0xAE, 0x6E, 0xEE, 0x1E, 0x9E, 0x5E, 0xDE, 0x3E, 0xBE, 0x7E, 0xFE,
  0x01, 0x81, 0x41, 0xC1, 0x21, 0xA1, 0x61, 0xE1, 0x11, 0x91, 0x51, 0xD1, 0x31, 0xB1, 0x71, 0xF1,
  0x09, 0x89, 0x49, 0xC9, 0x29, 0xA9, 0x69, 0xE9, 0x19, 0x99, 0x59, 0xD9, 0x39, 0xB9, 0x79, 0xF9, 
  0x05, 0x85, 0x45, 0xC5, 0x25, 0xA5, 0x65, 0xE5, 0x15, 0x95, 0x55, 0xD5, 0x35, 0xB5, 0x75, 0xF5,
  0x0D, 0x8D, 0x4D, 0xCD, 0x2D, 0xAD, 0x6D, 0xED, 0x1D, 0x9D, 0x5D, 0xDD, 0x3D, 0xBD, 0x7D, 0xFD,
  0x03, 0x83, 0x43, 0xC3, 0x23, 0xA3, 0x63, 0xE3, 0x13, 0x93, 0x53, 0xD3, 0x33, 0xB3, 0x73, 0xF3, 
  0x0B, 0x8B, 0x4B, 0xCB, 0x2B, 0xAB, 0x6B, 0xEB, 0x1B, 0x9B, 0x5B, 0xDB, 0x3B, 0xBB, 0x7B, 0xFB,
  0x07, 0x87, 0x47, 0xC7, 0x27, 0xA7, 0x67, 0xE7, 0x17, 0x97, 0x57, 0xD7, 0x37, 0xB7, 0x77, 0xF7, 
  0x0F, 0x8F, 0x4F, 0xCF, 0x2F, 0xAF, 0x6F, 0xEF, 0x1F, 0x9F, 0x5F, 0xDF, 0x3F, 0xBF, 0x7F, 0xFF
};

//----------deklaracje funkcji prywatnych----------------//
void LCD_wait(uint32_t period);
void LCD_write_cmd(uint8_t rs, uint8_t data);
void LCD_write_byte_to_section(uint16_t cs, uint8_t rs, uint8_t data);
//-----------------------------Funkcje publiczne---------//


/* 						 LCD_init
 * \opis Konfiguracja wstępna
 */ 
 
int8_t LCD_initialize(void)
{
    GPIO_InitTypeDef GPIO_Conf;
    uint8_t data[8];

    //włącz zegar GPIOA,GPIOB
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_GPIOB|RCC_APB2Periph_GPIOC, ENABLE);

    //linie jako wyjście
    GPIO_Conf.GPIO_Pin=D0|D1|D2|D3|D4|D5|D5|D6|D7;
    GPIO_Conf.GPIO_Mode=GPIO_Mode_Out_PP;
    GPIO_Conf.GPIO_Speed=GPIO_Speed_50MHz;
    GPIO_Init(GPIOA,&GPIO_Conf);
    GPIO_Conf.GPIO_Pin=CSA|CSB|RST_LCD;
    GPIO_Init(GPIOB,&GPIO_Conf);
    GPIO_Conf.GPIO_Pin=LCD_ENABLE|RW|RS;
    GPIO_Init(GPIOC,&GPIO_Conf);

    GPIO_Reset(GPIOA,D0|D1|D2|D3|D4|D5|D5|D6|D7);
    GPIO_Reset(GPIOB,CSA|CSB|RST_LCD);
    GPIO_Reset(GPIOC,RW|RS);
    GPIO_Set(GPIOC,LCD_ENABLE);

    //włacz wyświetlacz
    GPIO_Set(GPIOB,RST_LCD|CSA|CSB);    
    //Wzeruj pamięć
    LCD_ClearDisplayRAM();
    //włącz wyświetlanie
    LCD_write_cmd(LCD_CMD,LCD_DISP_ON);

    return 0;
}

/* 						 LCD_write_byte
 * \opis Wysyła 1 bajt do LCD
   \argument rs - wybór między wysłaniem danych a wysłaniem komendy
   \argument data - bajt do wysłania
 */

void LCD_write_cmd(uint8_t rs, uint8_t data)
{
  LCD_write_byte_to_section(LCD_LEFT_SECTION, rs, data);
  LCD_write_byte_to_section(LCD_MID_SECTION, rs, data);
  LCD_write_byte_to_section(LCD_RIGHT_SECTION, rs, data);
}

void LCD_set_ram_address(uint16_t address)
{
    uint16_t x_addr, y_addr;
    x_addr = (address / LCD_PAGE_SIZE);
    y_addr = (address % LCD_PAGE_SIZE);
    LCD_write_cmd(LCD_CMD, LCD_SET_X_ADDR + x_addr );
    LCD_write_cmd(LCD_CMD, LCD_SET_Y_ADDR + y_addr );
}

void LCD_write_buffer(uint8_t *buf, uint16_t size)
{
  uint32_t i,j,cntr,section;
  cntr = 0;

  for(i=0 ; i<LCD_NUM_PAGES ; i++)
  {
    LCD_set_ram_address(i*LCD_PAGE_SIZE);

    section = LCD_LEFT_SECTION;
    for(j=0 ; j < LCD_PAGE_SIZE ; j++)
      LCD_write_byte_to_section(section, LCD_DATA, buf[cntr++]);

    section = LCD_MID_SECTION;
    for(j=0 ; j < LCD_PAGE_SIZE ; j++)
      LCD_write_byte_to_section(section, LCD_DATA, buf[cntr++]);

    section = LCD_RIGHT_SECTION;
    for(j=0 ; j < LCD_PAGE_SIZE ; j++)
      LCD_write_byte_to_section(section, LCD_DATA, buf[cntr++]);
  }
}


void LCD_ClearDisplayRAM(void)
{
  uint32_t i,j;
  for(i=0 ; i<LCD_NUM_PAGES ; i++)
  {
    LCD_set_ram_address(i*LCD_PAGE_SIZE);
    for(j=0 ; j<LCD_PAGE_SIZE ; j++)
      LCD_write_cmd(LCD_DATA, 0);
  }
}

/* 						 LCD_wait
 * \opis Funckja Czeka 40us
 */ 
 
void LCD_wait(uint32_t period)
{
	while(period--)
	__asm("NOP");
}

void LCD_write_byte_to_section(uint16_t cs, uint8_t rs, uint8_t data)
{
    uint8_t tmp;

    GPIO_Reset(GPIOB, cs);

    GPIO_Reset(GPIOC,RW);
	
    //komenda / dane
	  if(rs)
        GPIO_Set(GPIOC,RS);
    else
        GPIO_Reset(GPIOC,RS);
    
    // bajt do wysłania
    tmp = BitReverseTable256[data];
    GPIO_Set(GPIOA,tmp);
    GPIO_Reset(GPIOA, (~tmp) & 0xff);

    //zapis
    LCD_wait(LCD_1_U);
    GPIO_Set(GPIOC,LCD_ENABLE);
    LCD_wait(LCD_1_U);
    GPIO_Reset(GPIOC,LCD_ENABLE);
    LCD_wait(LCD_500_N);
    
    GPIO_Set(GPIOB,CSB|CSA);
}
